@@ -116,6 +116,20 @@ namespace glz
       return t;
    }();
 
+   inline constexpr std::array<uint16_t, 256> char_escape_table = [] {
+      auto combine = [](const char chars[2]) -> uint16_t { return uint16_t(chars[0]) | (uint16_t(chars[1]) << 8); };
+
+      std::array<uint16_t, 256> t{};
+      t['\b'] = combine(R"(\b)");
+      t['\t'] = combine(R"(\t)");
+      t['\n'] = combine(R"(\n)");
+      t['\f'] = combine(R"(\f)");
+      t['\r'] = combine(R"(\r)");
+      t['\"'] = combine(R"(\")");
+      t['\\'] = combine(R"(\\)");
+      return t;
+   }();
+
    consteval uint32_t repeat_byte4(const auto repeat) { return uint32_t(0x01010101u) * uint8_t(repeat); }
 
    consteval uint64_t repeat_byte8(const uint8_t repeat) { return 0x0101010101010101ull * repeat; }
@@ -386,7 +400,7 @@ namespace glz
    }
 
    template <string_literal str, auto Opts>
-      requires(has_is_padded(Opts) && str.size() <= padding_bytes)
+      requires(check_is_padded(Opts) && str.size() <= padding_bytes)
    GLZ_ALWAYS_INLINE void match(is_context auto&& ctx, auto&& it, auto&&) noexcept
    {
       static constexpr auto S = str.sv();
@@ -399,7 +413,7 @@ namespace glz
    }
 
    template <string_literal str, auto Opts>
-      requires(!has_is_padded(Opts))
+      requires(!check_is_padded(Opts))
    GLZ_ALWAYS_INLINE void match(is_context auto&& ctx, auto&& it, auto&& end) noexcept
    {
       const auto n = size_t(end - it);
@@ -419,8 +433,7 @@ namespace glz
          ctx.error = error_code::unexpected_end;
       }
       else if (*it == '/') {
-         while (++it != end && *it != '\n')
-            ;
+         while (++it != end && *it != '\n');
       }
       else if (*it == '*') {
          while (++it != end) {
@@ -675,10 +688,10 @@ namespace glz
    }
 
    template <auto Opts>
-      requires(has_is_padded(Opts))
+      requires(check_is_padded(Opts))
    GLZ_ALWAYS_INLINE void skip_string(is_context auto&& ctx, auto&& it, auto&& end) noexcept
    {
-      if constexpr (!has_opening_handled(Opts)) {
+      if constexpr (!check_opening_handled(Opts)) {
          ++it;
       }
 
@@ -765,10 +778,10 @@ namespace glz
    }
 
    template <auto Opts>
-      requires(not has_is_padded(Opts))
+      requires(not check_is_padded(Opts))
    GLZ_ALWAYS_INLINE void skip_string(is_context auto&& ctx, auto&& it, auto&& end) noexcept
    {
-      if constexpr (!has_opening_handled(Opts)) {
+      if constexpr (!check_opening_handled(Opts)) {
          ++it;
       }
 
@@ -817,7 +830,7 @@ namespace glz
    }
 
    template <auto Opts, char open, char close, size_t Depth = 1>
-      requires(has_is_padded(Opts) && not bool(Opts.comments))
+      requires(check_is_padded(Opts) && not bool(Opts.comments))
    GLZ_ALWAYS_INLINE void skip_until_closed(is_context auto&& ctx, auto&& it, auto&& end) noexcept
    {
       size_t depth = Depth;
@@ -865,7 +878,7 @@ namespace glz
    }
 
    template <auto Opts, char open, char close, size_t Depth = 1>
-      requires(has_is_padded(Opts) && bool(Opts.comments))
+      requires(check_is_padded(Opts) && bool(Opts.comments))
    GLZ_ALWAYS_INLINE void skip_until_closed(is_context auto&& ctx, auto&& it, auto&& end) noexcept
    {
       size_t depth = Depth;
@@ -920,7 +933,7 @@ namespace glz
    }
 
    template <auto Opts, char open, char close, size_t Depth = 1>
-      requires(!has_is_padded(Opts) && not bool(Opts.comments))
+      requires(!check_is_padded(Opts) && not bool(Opts.comments))
    GLZ_ALWAYS_INLINE void skip_until_closed(is_context auto&& ctx, auto&& it, auto&& end) noexcept
    {
       size_t depth = Depth;
@@ -1004,7 +1017,7 @@ namespace glz
    }
 
    template <auto Opts, char open, char close, size_t Depth = 1>
-      requires(!has_is_padded(Opts) && bool(Opts.comments))
+      requires(!check_is_padded(Opts) && bool(Opts.comments))
    GLZ_ALWAYS_INLINE void skip_until_closed(is_context auto&& ctx, auto&& it, auto&& end) noexcept
    {
       size_t depth = Depth;
